package com.galaxis.manatee.task;

import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.service.LogDownloadService;
import com.galaxis.manatee.service.LogStandardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.PageRequest;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.List;

/**
 * @author kxuan
 */
@Service
@Slf4j
public class WorkHourStatisticsRecently {
    private final LogDownloadService logDownloadService;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final LogStandardService logStandardService;

    public WorkHourStatisticsRecently(LogDownloadService logDownloadService, ChuanyunUserCompanyDao chuanyunUserCompanyDao, LogStandardService logStandardService) {
        this.logDownloadService = logDownloadService;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.logStandardService = logStandardService;
    }

    @Scheduled(fixedDelay = 3600000L)
    public void updateHour() {
        dayLog();
        standardWorkHour();
        standardWeeklyWorkHour();
        standardMonthlyWorkHour();
    }

    /**
     * 更新最近一个月的日志信息
     */
    public void dayLog() {
        logDownloadService.saveChuanyunLogDO(true);
    }

    /**
     * 同步最近一个月每日工时
     */
    public void standardWorkHour() {
        log.info("开始最近每日工时标准化");
        var pageSize = 20;
        var page = 0;
        long startTime=Instant.now().getEpochSecond();
        while(true) {
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunUserCompanyList = chuanyunUserCompanyDao.findAllByStatus(1,pageable);
            if (page<=chuanyunUserCompanyList.getTotalPages()){
                page += 1;
                //逐个将员工每天工时进行统计汇总
                List<ChuanyunUserCompanyDO> userCompanyDOList = chuanyunUserCompanyList.getContent();
                //工时标准化
                userCompanyDOList.forEach(chuanyunUserCompanyDO -> logStandardService.standardHourByUserCompanyAndIsRecent(chuanyunUserCompanyDO,true));
            }else {
                break;
            }
        }
        log.info("最近每日工时标准化"+(Instant.now().getEpochSecond()-startTime)+"秒");
    }

    /**
     * 人员-项目-周度工时汇总
     */
    private void standardWeeklyWorkHour() {
        long start=Instant.now().getEpochSecond();
        //更新数据库
        log.info("开始最近周工时标准化");
        logStandardService.standardWeekMemberHourRecent();
        log.info("最近周工时标准化"+(Instant.now().getEpochSecond()-start)+"秒");
    }

    /**
     * 单月工时汇总
     */
    private void standardMonthlyWorkHour() {
        log.info("开始最近月工时汇总");
        var flag = true;
        var pageSize = 20;
        var page = 0;
        while(flag) {
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunUserCompanyList = chuanyunUserCompanyDao.findAllByStatus(1,pageable);
            if (page<=chuanyunUserCompanyList.getTotalPages()){
                page += 1;
                List<ChuanyunUserCompanyDO> userCompanyDOList = chuanyunUserCompanyList.getContent();
                //单项目单人单月成本
                userCompanyDOList.forEach(chuanyunUserCompanyDO ->  logStandardService.standardSelfCostByUserId(chuanyunUserCompanyDO.getUserId()));
            }else {
                flag = false;
            }
        }
    }
}
